;; $Revision: 1.1 $ $Date: 2005/07/14 04:11:44 $
;;; eggdoc-texinfo eggdoc

(use eggdoc)
(use eggdoc-zb)

(define doc
  `((eggdoc:begin
     (name "eggdoc-texinfo")
     (description "Render eggdoc source to .texi, .info and .pdf")
     (author (url ,zbigniew-homepage "Zbigniew"))
     (history
      (version "1.0" "Initial release"))

     (requires (url "sxml-transforms.html" "sxml-transforms")
               (url "sxml-tools.html" "sxml-tools"))

     (usage)
     (download "eggdoc-texinfo.egg")

     (documentation
      (p "eggdoc-texinfo produces Texinfo output for Chicken eggs.  You feed it plain old eggdoc SXML source, which it transforms into an intermediate representation of Texinfo in SXML, and then transforms that into " (tt ".texi") " output.  From there, you can use " (tt "makeinfo") " to produce an " (tt ".info") " file, and " (tt "texi2pdf") " to create a " (tt ".pdf") ".")

      (subsection "Your eggdoc and you"
        (group   ;; Can we get rid of this stupid group tag?  It annoys me.
         (procedure "(eggdoc->texinfo DOC [STYLESHEET])"
                    (p "Transform the SXML document " (tt "DOC") " into Texinfo SXML using the optional SXSLT stylesheet " (tt "STYLESHEET") ", then transform the result into Texinfo output.  If not specified, the built-in stylesheet will be used, which is also accessible with " (tt "(eggdoc:make-texinfo-stylesheet)") ".")
                    (p "If you currently use the optional stylesheet to convert eggdoc tags into other eggdoc expressions, via " (tt "*macro*") ", it will continue to work in the same way."))

         (procedure "(eggdoc:make-texinfo-stylesheet DOC)"
                    (p "Returns the default SXSLT stylesheet, which transforms eggdoc to Texinfo SXML.  This is a procedure because the stylesheet is built dynamically using the contents of the SXML document " (tt "DOC") ".")
                    (p "You can append to or modify this stylesheet and then pass it to " (tt "eggdoc->texinfo") " or " (tt "eggdoc->texinfo-sxml") "."))
         (procedure "(eggdoc->texinfo-sxml DOC [STYLESHEET])"
                    (p "Generate the intermediate SXML representation of Texinfo used by this egg.  You don't normally need to call this function except when debugging, or when you explicitly need to call " (tt "texinfo-sxml->texinfo") ".")
                    (p "Texinfo SXML was designed especially for this egg, and is not compatible with the " (q "Texinfo XML") " output of " (tt "makeinfo --xml") "."))
         (procedure "(texinfo-sxml->texinfo DOC [STYLESHEET])"
                    (p "Convert a Texinfo SXML document into Texinfo output suitable for writing to a " (tt ".texi") " file.  The default stylesheet is " (tt "universal-texi-conversion-rules") " and a modified version can be passed in if you feel the need to hack the " (tt ".texi") " output."))
         (stylesheet "universal-texi-conversion-rules"
                    (p "An SXSLT stylesheet used to convert Texinfo SXML into raw " (tt ".texi") " output."))
         (procedure "(eggdoc:make-defsig TAG SIG . [BODY])"
                    (p "Use this procedure inside a stylesheet to generate a new signature tag, just like "
                       (tt "procedure") " or " (tt "macro") ".  It expands to a "
                       (tt "(definition (signatures (signature ...)))") " expression.  For example,")
                    (pre "(eggdoc->texinfo doc `( ,@(eggdoc:make-stylesheet doc)\n"
                         "                        (constant *macro* . ,eggdoc:make-defsig)\n"
                         "                        (method   *macro* . ,eggdoc:make-defsig) ))")
                    (p "will create new " (tt "constant") " and " (tt "method") " tags."))))
      (subsection "Raising a happy, healthy child"
        (p "This extension is much more strict about well-formed eggdocs than the eggdoc extension itself, mostly because HTML parsers are so forgiving, and also because the stylesheet transformation is more complex.  In the course of converting many pre-existing eggdocs, I have come up with several tips to help you write correct documents and track down problems.")
        (ol
         (li "Sections must be nested in descending order, and levels may not be skipped: " (tt "section -> subsection -> subsubsection -> subsubsubsection") ".  HTML will just produce incorrect-looking output for bad nests, but Texinfo will barf because document nodes are not created correctly.  This goes double for the " (tt "documentation, example") " and " (tt "license") " tags; these are all " (tt "section") " tags at heart and the next nesting level is a subsection: " (tt "(documentation (subsection ...))") ".")
         (li "Certain tags, such as " (tt "tt") " and " (tt "em") ", have Texinfo representations that don't allow embedded newlines (paragraphs).  In other words, you may not say " (tt "(b (p ...) (p ...))") (& "mdash") "instead use " (tt "(p (b ...) (b ...))") ".  And you cannot say " (tt "(tt \"line1 \\n line2\")") ", which doesn't render correctly in HTML anyway"  (& "mdash") "use " (tt "(pre ...)") ".")
         (li "Try to avoid extra whitespace within strings.  Info will usually render the extra space, although HTML and TeX ignore it.  I consider this a problem with Info, but regardless it is safest to avoid unclosed multi-line strings.")
         (li "The parser doesn't automatically translate \"straight quotes\" to " (q "smart quotes.") "  To get smart quotes, use the " (tt "(q ...)") " tag, which also works in HTML.")
         (li "Definition signatures are parsed as Scheme expressions, and any operator name pulled out.  Good output is usually generated even for illegal expressions (with a warning issued).  However, read syntax confuses the parser; use the " (tt "read-syntax") " definition tag, which disables parsing, in this case.  See the objc egg for an example.")
         (li "Texinfo doesn't flow @verbatim (pre) tags or function definition lines.  Long lines will need to be reformatted manually for PDF output.")
         (li (div "Bare strings should not be used in section bodies" (& "mdash") "wrap in a block tag such as " (tt "p") ".  It works in HTML, but runstogether in Texinfo."
              (pre "(section \"License\" \"Public domain\")     ;; invalid\n"
                   "(section \"License\" (p \"Public domain\")) ;; correct")))

         ))
      (subsection "Batch conversion"
        (p "Inside the egg under " (tt "extras/") " I included some simple scripts to help with conversion of one or more eggdoc sources to Texinfo.")
        (dl
         (dt (tt "eggdoc-texinfo-shim.scm")) (dd "Overrides standard eggdoc functions with eggdoc-texinfo versions, so you do not have to modify the source.")
         (dt (tt "eggdoc-html-to-texi.sh"))  (dd "Converts standard eggdoc source to .texi, .info and .pdf output.")
         (dt (tt "batch-convert.sh"))        (dd "Takes a file containing a list of eggdocs and converts each one.")
         (dt (tt "install-info.sh"))         (dd "Install .info file(s) into your Info directory."))

        (p "I converted about a hundred eggs using this process.  Browse the "
           (url "http://3e8.org/zb/eggs/eggdoc-texinfo/output" ".texi, .info and .pdf output") " or "
           (url "http://3e8.org/zb/eggs/eggdoc-texinfo/output/eggdoc-texinfo-output.tgz" "download the entire thing") ".")))

     (license
      ,licenses:zbigniew-bsd))))

(eggdoc->html doc `( ,@(eggdoc:make-stylesheet doc)
                     (stylesheet *macro* . ,eggdoc:make-defsig)))